#include "PokittoPlus.h"
#include "ESPLoader.h"
#include "Popup.h"

bool PokittoPlus::update()
{
    if(Pokitto::Buttons::repeat(BTN_UP,2))
    {
        if(m_selected > 0 )
           m_selected --;
    }
    else if(Pokitto::Buttons::repeat(BTN_DOWN,2))
    {
         if(m_selected < 3 )
           m_selected ++;
    }
    
    if(Pokitto::Buttons::pressed(BTN_A))
    {
        switch(m_selected)
        {
        case 0:
            bootProgramingMode();
            break;
        case 1:
            bootNormalMode();
            break;
        case 2:
            reboot();
            break;
        case 3:
            Popup::Error("Error", "Not implemented");
            break;

        }
    }
    else if(Pokitto::Buttons::pressed(BTN_B))
        return false;
    return true;
}

void PokittoPlus::draw()
{
    Pokitto::Display::setColor(CAYAN);
    Pokitto::Display::fillRect(0,STATUSBARHEIGHT,Pokitto::Display::width,Pokitto::Display::height-STATUSBARHEIGHT);

    const std::string entries[]= {"Boot in programming mode", "Boot in normal mode", "Reboot", "Info"};
    //const unsigned char *icons [] = {Icons::EEPROM::CookiesManager, Icons::EEPROM::Backup, Icons::EEPROM::Restore, Icons::EEPROM::Format, Icons::EEPROM::HexEdit};

    for(auto i = 0 ;i < 4; i++)
    {
        Pokitto::Display::setColor(WHITE);

        if(m_selected==i)
            Pokitto::Display::setColor(LIGHT_BLUE);

        Pokitto::Display::fillRect(5,STATUSBARHEIGHT+5+i*20,Pokitto::Display::width-10,20-2);
        if(m_selected==i)
            Pokitto::Display::setColor(WHITE);
        else
            Pokitto::Display::setColor(BLACK);

        Pokitto::Display::setCursor(35, STATUSBARHEIGHT+11+i*20);
        Pokitto::Display::print(entries[i].c_str());

        //Pokitto::Display::drawBitmap(10,STATUSBARHEIGHT+7+i*20,icons[i]);
    }
}

bool PokittoPlus::flashFirmware(std::string path, uint32_t flash_offset)
{
    FileHandle *file=sdFs->open(path.c_str(), O_RDWR );
    
    if(!file)
    {
        Popup::Error("Error","File open failed");
        return false;
    }
    ESPLoader Loader(230400);//460800

    uint32_t fsize=file->flen();
    Popup::Loading("Connecting to ESP8266 Module");
    Loader.enterBootLoader();
    wait_ms(1000);

    if(Loader.sync())
    {
        if(Loader.flash_begin(fsize, flash_offset))
        {
            uint8_t data[Loader.FLASH_WRITE_SIZE];
            uint32_t parts=(fsize+Loader.FLASH_WRITE_SIZE-1)/Loader.FLASH_WRITE_SIZE;
            for(auto i=0;i<parts;i++)
            {
                Popup::progressBar("Flashing Firmware", (100*i)/parts);
                uint32_t count=file->read(data, Loader.FLASH_WRITE_SIZE);
                if(!Loader.flash_block(data, i, count ))
                {
                    Popup::Error("Error","Sending data to ESP8266 Module Failed");
                    return false;
                }
            }
            file->close();
            Loader.flash_end(true);

            Popup::InfoBox("Flashing ESP8266","Firmware flashed Successfully");
            return true;
            
        }
        else
           Popup::Error("Error","Flash Erase Failed");
        
    }
    else
        Popup::Error("Error","Can't connect ESP8266 Module");
        
    return false;
}

void PokittoPlus::reboot()
{
    DigitalOut esp_pinEnable(P0_21);
    DigitalOut esp_pinReset(P0_20);
    DigitalOut esp_pinProg(P1_1);
    
    Popup::Loading("Rebooting ESP8266 Module");
    esp_pinEnable=0;
    wait_ms(1000);
    esp_pinEnable=1;
    Popup::InfoBox("Rebooting ESP8266","ESP8266 Rebooted Successfully");
}

void PokittoPlus::bootProgramingMode()
{
    DigitalOut esp_pinEnable(P0_21);
    DigitalOut esp_pinReset(P0_20);
    DigitalOut esp_pinProg(P1_1);
    
    Popup::Loading("Booting ESP8266 in Programming mode");
    esp_pinEnable = 0;
	esp_pinProg = 0;
	esp_pinReset = 1;
	wait_ms(1000);
	esp_pinEnable = 1;
    Popup::InfoBox("Booting ESP8266","ESP8266 is in programming mode");
}

void PokittoPlus::bootNormalMode()
{
    DigitalOut esp_pinEnable(P0_21);
    DigitalOut esp_pinReset(P0_20);
    DigitalOut esp_pinProg(P1_1);
    
    Popup::Loading("Booting ESP8266 in Normal mode");
    esp_pinEnable = 0;
	esp_pinProg = 1;
	esp_pinReset = 1;
	wait_ms(1000);
	esp_pinEnable = 1;
    Popup::InfoBox("Booting ESP8266","ESP8266 is in normal mode");
}