#pragma once
#include "SDBrowse.h"
#include "Icons.h"
#include "Pop.h"
#include "Popup.h"
#include "Bmp.h"
#include "Settings.h"
#include "PokittoPlus.h"

SDBrowse::SDBrowse(): m_curDir("")
{

}

void SDBrowse::init()
{
    Popup::Loading("Loading SD Card..");
    m_itemcount=fileCount(m_curDir);
    m_loadItems();
}

size_t SDBrowse::fileCount(std::string path)
{
   m_redraw=true;
   auto c=0;
   DirHandle *dir = sdFs->opendir(path.c_str());
   while(dir->readdir())
       c++;
    dir->closedir();
   return c;
}

bool SDBrowse::update()
{
    if(menustate==MenuState::SoundPlayer)
    {
        if(!soundplayer->update())
        {
            delete soundplayer;
            menustate=MenuState::MainMenu;
        }
        return true;
    }

    if(Pokitto::Buttons::repeat(BTN_RIGHT,5) && m_itemcount > 5)
    {
        if(m_beginitem < m_itemcount-5 )
        {
            m_beginitem+=5;
            m_selecteditem=0;
            m_loadItems();
            m_drawAllItems();
            m_drawScrollbar();
            //m_redraw=true;
        }
        return true;
    }

    if(Pokitto::Buttons::repeat(BTN_LEFT,5) && m_itemcount > 5)
    {
        if(m_beginitem > 0)
        {
            m_beginitem -= 5;
            m_selecteditem=0;
            m_loadItems();
            m_drawAllItems();
            m_drawScrollbar();
            //m_redraw=true;
        }
        return true;
    }

    if(Pokitto::Buttons::repeat(BTN_UP,7))
    {
        if(m_selecteditem==0)
        {
            if(m_beginitem >= 5)
            {
                m_selecteditem=4;
                m_beginitem-=5;
                m_loadItems();
                m_drawAllItems();
                m_drawScrollbar();
                //m_redraw=true;
            }
            else
                return true;
        }
        else
        {
            m_selecteditem--;
            m_drawItem(m_selecteditem+1);
            m_drawItem(m_selecteditem);
        }

       return true;
    }

    if(Pokitto::Buttons::repeat(BTN_DOWN,7))
    {
        if(m_selecteditem+m_beginitem >= m_itemcount-1)
            return true;

        m_selecteditem++;

        if(m_selecteditem == 5)
        {
            m_selecteditem=0;
            m_beginitem+=5;
            m_loadItems();
            m_drawAllItems();
            m_drawScrollbar();
            //m_redraw=true;
        }
        else
        {
            m_drawItem(m_selecteditem-1);
            m_drawItem(m_selecteditem);
        }

        return true;
    }

    if(Pokitto::Buttons::pressed(BTN_A))
    {
        if(m_Items[m_selecteditem].Type==ItemType::Folder)
        {
            m_curDir=m_Items[m_selecteditem].fullPath;

            m_pushPrev();
            m_beginitem=0;
            m_selecteditem=0;

            m_itemcount=fileCount(m_curDir);
            m_loadItems();
            m_redraw=true;
        }else
        {

            if(m_Items[m_selecteditem].Type==ItemType::Pop)
            {
                while(true)
                {
                    auto ret=Popup::PopInfo(m_Items[m_selecteditem].fullPath);
                    if(ret==0)
                        break;
                    if(ret==2) // C button
                    {
                        size_t num=0;
                        if(Pop::drawScreenShoot(num++))
                        {
                            while(true)
                            {
                                Pokitto::Buttons::update();

                                if(Pokitto::Buttons::pressed(BTN_A))
                                    if(!Pop::drawScreenShoot(num++))
                                        break;
                                if(Pokitto::Buttons::pressed(BTN_B))
                                    break;

                            }
                        }

                    }
                    else if(ret==1) //A button
                        Settings::jumpToLoader(std::string("Load:"+m_Items[m_selecteditem].fullPath).c_str());
                }
            }
            else if(m_Items[m_selecteditem].Type==ItemType::BinGame)
            {
                 if(Popup::BinInfo(m_Items[m_selecteditem].fullPath))
                        Settings::jumpToLoader(std::string("Load:"+m_Items[m_selecteditem].fullPath).c_str());
            }
            else if(m_Items[m_selecteditem].Type==ItemType::Gameboy)
            {
                 if(Popup::BinInfo(m_Items[m_selecteditem].fullPath))
                        Settings::jumpToLoader(std::string("Load:"+m_Items[m_selecteditem].fullPath).c_str());
            }
            else if(m_Items[m_selecteditem].Type==ItemType::Der)
            {
                if(Popup::LoaderInfo(m_Items[m_selecteditem].fullPath))
                {

                    FileHandle *loaderfile=sdFs->open(m_Items[m_selecteditem].fullPath.c_str(),O_RDONLY);
                    if(!loaderfile)
                        return false;

                    size_t fsize=loaderfile->flen();
                    loaderfile->lseek(fsize-4,SEEK_SET);
                    //size_t version= 0;
                    //loaderfile->read(&version,4);
                    size_t jumpadd=0;
                    loaderfile->read(&jumpadd,4);
                    loaderfile->lseek(0,SEEK_SET);
                    if (fsize>0x40000-jumpadd)
                        fsize = 0x40000-jumpadd;

                    uint32_t counter=0;
                    uint8_t data[0x100];


                    while (1) {
                            if(counter==0)
                                Popup::Loading("Erasing old loader...");
                            else
                                Popup::progressBar("Flashing Loader",(counter*100)/fsize);
                        if (counter >= fsize) {
                            break;
                        }
                        if (loaderfile->read(&data[0],0x100)<0x100) {
                            if (fsize-counter>0x100) {
                                    Popup::Error("Error", "Reading Loader file");
                                    return true;
                            }
                        }

                        if (CopyPageToFlash(jumpadd+counter, reinterpret_cast<std::uint8_t*>(data))){
                            Popup::Error("Error","Flash write error");
                            return true;
                        } else {
                            counter += 0x100;
                        }
                    }

                    Popup::InfoBox("Loader","Loader installed successfully");
                }
            }else if(m_Items[m_selecteditem].Type==ItemType::Snd || m_Items[m_selecteditem].Type==ItemType::Wav)
            {
                soundplayer=new SoundPlayer(m_Items[m_selecteditem].fullPath);
                menustate=MenuState::SoundPlayer;
            }
            else if(m_Items[m_selecteditem].Type==ItemType::Bmp)
            {
                if(Bmp::draw(m_Items[m_selecteditem].fullPath))
                {
                    while(true)
                    {
                        Pokitto::Buttons::update();
                        if(Pokitto::Buttons::pressed(BTN_A) || Pokitto::Buttons::pressed(BTN_B))
                            break;
                    }
                }

            }
            else if(m_Items[m_selecteditem].Type==ItemType::EspFirm)
            {
                 if(Popup::BinInfo(m_Items[m_selecteditem].fullPath))
                        PokittoPlus::flashFirmware(m_Items[m_selecteditem].fullPath);
            }
            else
                Popup::fileInfo(m_Items[m_selecteditem].fullPath);

            m_redraw=true;
        }

    }else if(Pokitto::Buttons::pressed(BTN_B))
    {
        if(m_curDir=="")
            return false;

        auto pos = m_curDir.find_last_of('/');
        m_curDir = m_curDir.substr(0,pos);

        m_pullPrev();
        m_itemcount=fileCount(m_curDir);
        m_loadItems();
        m_redraw=true;

    }else if(Pokitto::Buttons::pressed(BTN_C))
    {
        if(m_movemode)
        {
            if(Popup::YesNo("Are you sure?","Do you want to move file here:\n"+m_movepath))
            {
                std::string moveto=m_curDir + "/" + m_movepath.substr(m_movepath.find_last_of('/')+1,m_movepath.size());
                FRESULT res = f_rename(m_movepath.c_str(), moveto.c_str());
                if( res != FR_OK)
                    Popup::Error("Error ","moving file\n"+moveto, res);
                else
                {
                    m_itemcount=fileCount(m_curDir);
                    m_movemode=false;
                }

            }
            m_redraw=true;
            return true;
        }

        DirHandle *dir = sdFs->opendir(m_curDir.c_str());
        for(auto j=0;j<m_beginitem+m_selecteditem;j++)
            dir->readdir();

        std::string fileName = m_curDir + "/";
        fileName += dir->readdir()->d_name;
        dir->closedir();

        std::string filemenu[]={"Info", "Open as","Copy", "Move", "Delete", "Rename", "New file", "New folder"};
        const unsigned char * menuicons[]={Icons::Actions::Info, Icons::Actions::Open, Icons::Actions::Copy, Icons::Actions::Move, Icons::Actions::DeleteFile, Icons::Actions::Rename, Icons::Actions::NewFile, Icons::Actions::NewFolder};
        size_t ret=Popup::Menu("File menu",filemenu, menuicons, 8);

        switch(ret)
        {
        case 0:
            Popup::fileInfo(fileName);
            break;
        case 1:
            Popup::Error("Error","Not implemented");
            break;
        case 2:
            Popup::Error("Error","Not implemented");
            break;
        case 3:
            m_movepath=fileName;
            m_movemode=true;

            break;
        case 4:

            if(Popup::YesNo("Are you sure?","Do you want to delete:\n"+fileName))
            {
                FRESULT res = f_unlink(fileName.c_str());
                if( res != FR_OK)
                    Popup::Error("Error ","deleting file\n"+fileName, res);
            }

            break;
        case 5:
            {
                std::string strname = fileName.substr(fileName.find_last_of('/')+1,fileName.size());
                char name[30];
                strcpy(name,strname.c_str());
                Popup::keyboard("enter file name", name, 30);
                if(strlen(name)==0)
                    break;
                FRESULT res = f_rename(fileName.c_str(), std::string(m_curDir+"/"+name).c_str());
                if( res != FR_OK)
                    Popup::Error("Error ","renaming file\n"+fileName, res);
            }

            break;
        case 6:
            {

                char name[30];
                name[0]=0;
                Popup::keyboard("enter file name", name, 30);
                if(strlen(name)==0)
                    break;
                std::string file = m_curDir + "/" + name;
                FileHandle *f=sdFs->open(file.c_str(), O_CREAT | O_WRONLY);
                if( !f)
                    Popup::Error("Error ","file create error\n"+file);
                f->close();
                m_itemcount=fileCount(m_curDir);
            }

            break;

        case 7:
            {
                char name[30];
                sprintf(name,"New Folder");
                Popup::keyboard("enter folder name", name, 30);
                if(strlen(name)==0)
                    break;
                std::string folder = m_curDir + "/" + name;
                FRESULT res=f_mkdir(folder.c_str());
                if( res != FR_OK)
                    Popup::Error("Error ","mkdir error\n"+folder, res);
                else
                    m_itemcount=fileCount(m_curDir);
            }

            break;

        case -1:
            break;
        };

        m_redraw=true;
    }
    return true;
}

void SDBrowse::draw()
{
    if(menustate==MenuState::SoundPlayer)
    {
        soundplayer->draw();
        return;
    }

    if(!m_redraw)
        return;

    Pokitto::Display::setColor(CAYAN);
    Pokitto::Display::fillRect(0,STATUSBARHEIGHT,Pokitto::Display::width,Pokitto::Display::height-STATUSBARHEIGHT);

    m_drawScrollbar();

    Pokitto::Display::setColor(DARK_GRAY);
    Pokitto::Display::fillRect(0,STATUSBARHEIGHT,Pokitto::Display::width,12);

    //DWORD fre_clust;
    //FATFS *fs=&sdFs->_fs;
    //f_getfree(0, &fre_clust, &fs);

    //Pokitto::Display::setColor(GREEN);
    //Pokitto::Display::fillRect(0,STATUSBARHEIGHT+12,Pokitto::Display::width*(sdFs->_fs.n_fatent-fre_clust)/sdFs->_fs.n_fatent,1);

    Pokitto::Display::setFont(font5x7);
    Pokitto::Display::setCursor(15,STATUSBARHEIGHT+2);
    Pokitto::Display::setColor(WHITE);
    Pokitto::Display::print(m_curDir.c_str());
    Pokitto::Display::print("/");

    if(m_movemode)
    {
        Pokitto::Display::setColor(WHITE);
        Pokitto::Display::fillRect(Pokitto::Display::width-15,STATUSBARHEIGHT,12,12);
        Pokitto::Display::drawBitmap(Pokitto::Display::width-13,STATUSBARHEIGHT+2,Icons::Actions::Move);
    }
    if(m_itemcount==0)
    {
        Pokitto::Display::setColor(WHITE);
        Pokitto::Display::setCursor(46,STATUSBARHEIGHT+25);
        Pokitto::Display::print("This Folder Is Empty");

        return;
    }
    m_drawAllItems();

    m_redraw=false;
}

void SDBrowse::m_pushPrev()
{
    m_prevSelecteditem[m_pPrev]=m_selecteditem;
    m_prevBeginitem[m_pPrev]=m_beginitem;
    if(m_pPrev < 8)
        m_pPrev++;
}

void SDBrowse::m_pullPrev()
{
    if(m_pPrev > 0)
        m_pPrev--;

    m_selecteditem=m_prevSelecteditem[m_pPrev];
    m_beginitem=m_prevBeginitem[m_pPrev];
}


void SDBrowse::m_drawAllItems()
{

    for(size_t i=0;i<m_itemlistcount;i++)
        m_drawItem(i);

    for(size_t i=m_itemlistcount;i<5;i++)
    {
        Pokitto::Display::setColor(CAYAN);
        Pokitto::Display::fillRect(5,STATUSBARHEIGHT+15+i*30,Pokitto::Display::width-10,30-2);
    }

}

void SDBrowse::m_drawItem(size_t num)
{
    Pokitto::Display::setColor(WHITE);
    if(m_selecteditem==num)
        Pokitto::Display::setColor(LIGHT_BLUE);

    Pokitto::Display::fillRect(5,STATUSBARHEIGHT+15+num*30,Pokitto::Display::width-10,30-2);

    if(m_Items[num].Type==ItemType::Folder)
        Pokitto::Display::drawBitmap(7,STATUSBARHEIGHT+15+2+num*30,Icons::Files::Folder);
    else
    {
        switch(m_Items[num].Type)
        {
            case ItemType::Bin:
                Pokitto::Display::drawBitmap(7,STATUSBARHEIGHT+15+2+num*30,Icons::Files::Bin);
                break;
            case ItemType::BinGame:
                Pokitto::Display::drawBitmap(7,STATUSBARHEIGHT+15+2+num*30,Icons::Files::PokiBin);
                break;
            case ItemType::Snd:
                Pokitto::Display::drawBitmap(7,STATUSBARHEIGHT+15+2+num*30,Icons::Files::Snd);
                break;
            case ItemType::Wav:
                Pokitto::Display::drawBitmap(7,STATUSBARHEIGHT+15+2+num*30,Icons::Files::Wav);
                break;
            case ItemType::Txt:
                Pokitto::Display::drawBitmap(7,STATUSBARHEIGHT+15+2+num*30,Icons::Files::Txt);
                break;
            case ItemType::Rom:
                Pokitto::Display::drawBitmap(7,STATUSBARHEIGHT+15+2+num*30,Icons::Files::Rom);
                break;
            case ItemType::Der:
                Pokitto::Display::drawBitmap(7,STATUSBARHEIGHT+15+2+num*30,Icons::Files::Der);
                break;
            case ItemType::Bmp:
                Pokitto::Display::drawBitmap(7,STATUSBARHEIGHT+15+2+num*30,Icons::Files::Bmp);
                break;
            case ItemType::Gameboy:
                Pokitto::Display::drawBitmap(7,STATUSBARHEIGHT+15+2+num*30,Icons::Files::Gb);
                break;
            case ItemType::EspFirm:
                Pokitto::Display::drawBitmap(7,STATUSBARHEIGHT+15+2+num*30,Icons::Files::EspFirm);
                break;
            case ItemType::File:
                Pokitto::Display::drawBitmap(7,STATUSBARHEIGHT+15+2+num*30,Icons::Files::File);
                break;
        }

        Pokitto::Display::setFont(font3x5);

        char s[10];
        sprintf(s,"%d.%01d KB",m_Items[num].Size/1024,(m_Items[num].Size%1024)/102);
        Pokitto::Display::setCursor(Pokitto::Display::width-10-strlen(s)*4,STATUSBARHEIGHT+15+21+num*30);
        if(m_selecteditem==num)
            Pokitto::Display::setColor(YELLOW);
        else
            Pokitto::Display::setColor(GREEN);
        Pokitto::Display::print(s);

        Pokitto::Display::setColor(DARK_GRAY);

        if(m_Items[num].Type==ItemType::Pop)
        {
            if(m_Items[num].hasIcon)
            {
                Pokitto::Display::setInvisibleColor(0xff);
                Pokitto::Display::drawBitmapData(7,STATUSBARHEIGHT+15+2+num*30, 24, 24, m_Items[num].Icon);
                Pokitto::Display::setInvisibleColor(PINK);
            }

            else
                Pokitto::Display::drawBitmap(7,STATUSBARHEIGHT+15+2+num*30, Icons::Files::Pop);

            Pokitto::Display::setColor(BLACK);
            Pokitto::Display::drawRect(6, STATUSBARHEIGHT+15+1+num*30, 25, 25);
            Pokitto::Display::setColor(PURPLE);
            Pokitto::Display::setCursor(35,STATUSBARHEIGHT+15+21+num*30);
            Pokitto::Display::print("by ");
            Pokitto::Display::print(m_Items[num].author.c_str());
        }

        Pokitto::Display::setFont(font5x7);
    }
    if(m_selecteditem==num)
        Pokitto::Display::setColor(WHITE);
    else
        Pokitto::Display::setColor(BLACK);

    Pokitto::Display::setCursor(35,STATUSBARHEIGHT+15+2+num*30);
    Pokitto::Display::print(m_Items[num].displayName.c_str());
}

void SDBrowse::m_loadItems()
{
    m_itemlistcount=0;
    if(m_itemcount < 1)
        return;
    size_t itemcount=5;
    if(m_itemcount-m_beginitem < 5)
        itemcount = m_itemcount-m_beginitem;

    FATFS_DIR dir;
    static FILINFO finfo;
    FRESULT res=f_opendir(&dir, m_curDir.c_str());
    if(res!=FR_OK)
        return;

    for(size_t j=0;j<m_beginitem;j++)
        res = f_readdir(&dir, &finfo);

    char nameBuffer[_MAX_LFN+1];
    finfo.lfname=nameBuffer;
    finfo.lfsize=_MAX_LFN;

    for(size_t i=0;i<itemcount;i++)
    {
        char *fileName=0;
        res = f_readdir(&dir, &finfo);
        if(res!=FR_OK || finfo.fname[0]==0)
            break;
        if(finfo.lfname[0]!=0)
            fileName=finfo.lfname;
        else
            fileName=finfo.fname;

        m_itemlistcount++;
        m_Items[i].displayName = fileName;
        m_Items[i].fullPath = m_curDir+"/"+fileName;

        static FILINFO info;
        f_stat(m_Items[i].fullPath.c_str(), &info);
        m_Items[i].Size=info.fsize;

        if(info.fattrib & AM_DIR)
            m_Items[i].Type=ItemType::Folder;
        else
        {
            char ext[4];
            char *extstr=strrchr(fileName,'.')+1;
            for(auto k=0;k<3;k++)
                ext[k]=tolower(*(extstr+k));
            ext[3]=0;

            if(strcmp(ext,"bin")==0)
            {
                FileHandle *fbin=sdFs->open(m_Items[i].fullPath.c_str(), O_RDONLY);
                if(fbin)
                {
                    uint32_t magic=0;
                    fbin->read(&magic,4);
                    if(magic==0X10008000)
                      m_Items[i].Type=ItemType::BinGame;
                    else
                      m_Items[i].Type=ItemType::Bin;
                    fbin->close();
                }
                else
                    m_Items[i].Type=ItemType::Bin;
            }
            else if(strcmp(ext,"pop")==0)
                m_Items[i].Type=ItemType::Pop;
            else if(strcmp(ext,"gb")==0)
                m_Items[i].Type=ItemType::Gameboy;
            else if(strcmp(ext,"snd")==0 || strcmp(ext,"raw")==0)
                m_Items[i].Type=ItemType::Snd;
            else if(strcmp(ext,"wav")==0)
                m_Items[i].Type=ItemType::Wav;
            else if(strcmp(ext,"txt")==0)
                m_Items[i].Type=ItemType::Txt;
            else if(strcmp(ext,"rom")==0)
                m_Items[i].Type=ItemType::Rom;
            else if(strcmp(ext,"der")==0)
                m_Items[i].Type=ItemType::Der;
            else if(strcmp(ext,"bmp")==0)
                m_Items[i].Type=ItemType::Bmp;
            else if(strcmp(ext,"esp")==0)
                m_Items[i].Type=ItemType::EspFirm;
            else
                m_Items[i].Type=ItemType::File;

            if(m_Items[i].Type==ItemType::Pop)
            {
                Pop::open(m_Items[i].fullPath);
                if(Pop::findTag(PopTag::TAG_IMG_24X24_4BPP))
                {
                    m_Items[i].hasIcon=true;
                    Pop::popFile->read(m_Items[i].Icon, 24*12);
                }
                else
                    m_Items[i].hasIcon=false;

                m_Items[i].author=Pop::getString(PopTag::TAG_AUTHOR);

                Pop::close();

            }

        }

        if(m_Items[i].displayName.size() > 30)
        {
            m_Items[i].displayName=m_Items[i].displayName.substr(0, 28);
            m_Items[i].displayName+="..";
        }
    }
}

 void SDBrowse::m_drawScrollbar()
 {
    Pokitto::Display::setColor(DARK_GRAY);
    Pokitto::Display::fillRect(Pokitto::Display::width-3,STATUSBARHEIGHT+16,2,145);

    if(m_itemcount > 5)
    {
        Pokitto::Display::setColor(WHITE);
        Pokitto::Display::fillRect(Pokitto::Display::width-3,STATUSBARHEIGHT+16+(145/((m_itemcount+4)/5))*m_beginitem/5,2,145/((m_itemcount+4)/5));
    }
 }
